/*

Fenryxo's library :-)

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

If this file is built as a shared library it's released under GNU LGPL 3+ and 2-Clause BSD.
Otherwise it's released under GNU GPL 3+.
 
This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See <http://www.gnu.org/licenses/> for more details.
*/

namespace Fenryxo.Logger{
	private const string LOG_DOMAIN = "libfenryxo";
	public static string domain;
	public static LogLevelFlags level;
	
	public static void init(string domain, LogLevelFlags level=LogLevelFlags.LEVEL_DEBUG){
		Fenryxo.Logger.domain = domain;
		Fenryxo.Logger.level = level;
		GLib.Log.set_default_handler(Logger.log_handler);
	}
	
	public static void log_handler(string? domain, LogLevelFlags level, string message){
		if(level > Fenryxo.Logger.level) return;
		stdout.printf("%10.10s ", domain ?? "<unknown>");
		
		string name = "";
		switch (level) {
			case LogLevelFlags.FLAG_RECURSION:
				name = "Recursion";
				break;
			case LogLevelFlags.FLAG_FATAL:
				name = "Fatal";
				break;
			case LogLevelFlags.LEVEL_CRITICAL:
				name = "Critical";
				break;
			case LogLevelFlags.LEVEL_ERROR:
				name = "Error";
				break;
			case LogLevelFlags.LEVEL_WARNING:
				name = "Warning";
				break;
			case LogLevelFlags.LEVEL_MESSAGE:
			case LogLevelFlags.LEVEL_INFO:
				name = "Info";
				break;
			case LogLevelFlags.LEVEL_DEBUG:
				name = "Debug";
				break;
			case LogLevelFlags.LEVEL_MASK:
				name = "Mask";
				break;
			default:
				name = "Unknown";
				break;
		}
		stdout.printf ("%8.8s ", name);
		stdout.printf(message);
		stdout.printf("\n");
		
	}
	
	
	/**
	 * Logs debug message with domain specified in Logger.domain.
	 * @param string format string
	 */
	public static void log_debug(string format, ...){
		va_list args = va_list();
		GLib.logv(Fenryxo.Logger.domain, LogLevelFlags.LEVEL_DEBUG, format, args);
	}
	
	/**
	 * Logs info message with domain specified in Logger.domain.
	 * @param string format string
	 */
	public static void log_message(string format, ...){
		va_list args = va_list();
		GLib.logv(Fenryxo.Logger.domain, LogLevelFlags.LEVEL_MESSAGE, format, args);
	}
	
	/**
	 * Logs warning with domain specified in Logger.domain.
	 * @param string format string
	 */
	public static void log_warning(string format, ...){
		va_list args = va_list();
		GLib.logv(Fenryxo.Logger.domain, LogLevelFlags.LEVEL_WARNING, format, args);
	}
	
	/**
	 * Logs critical warning with domain specified in Logger.domain.
	 * @param string format string
	 */
	public static void log_critical(string format, ...){
		va_list args = va_list();
		GLib.logv(Fenryxo.Logger.domain, LogLevelFlags.LEVEL_CRITICAL, format, args);
	}
	
	/**
	 * Logs error message with domain specified in Logger.domain.
	 * @param string format string
	 */
	public static void log_error(string format, ...){
		va_list args = va_list();
		GLib.logv(Fenryxo.Logger.domain, LogLevelFlags.LEVEL_ERROR, format, args);
	}
	
	/**
	 * Logs  message with domain specified in Logger.domain.
	 * 
	 * @param level Level of message
	 * @param string format string
	 */
	public static void log_with_level(LogLevelFlags level, string format, ...){
		va_list args = va_list();
		GLib.logv(Fenryxo.Logger.domain, level, format, args);
	}
	
	
	/**
	 * Logs debug message with libfenryxo domain
	 * @param string format string
	 */
	private static void lib_debug(string format, ...){
		va_list args = va_list();
		GLib.logv(LOG_DOMAIN, LogLevelFlags.LEVEL_DEBUG, format, args);
	}
	
	/**
	 * Logs info message with libfenryxo domain
	 * @param string format string
	 */
	private static void lib_message(string format, ...){
		va_list args = va_list();
		GLib.logv(LOG_DOMAIN, LogLevelFlags.LEVEL_MESSAGE, format, args);
	}
	
	/**
	 * Logs warning with libfenryxo domain
	 * @param string format string
	 */
	private static void lib_warning(string format, ...){
		va_list args = va_list();
		GLib.logv(LOG_DOMAIN, LogLevelFlags.LEVEL_WARNING, format, args);
	}
	
	/**
	 * Logs critical warning with libfenryxo domain
	 * @param string format string
	 */
	private static void lib_critical(string format, ...){
		va_list args = va_list();
		GLib.logv(LOG_DOMAIN, LogLevelFlags.LEVEL_CRITICAL, format, args);
	}
	
	/**
	 * Logs error message with libfenryxo domain
	 * @param string format string
	 */
	private static void lib_error(string format, ...){
		va_list args = va_list();
		GLib.logv(LOG_DOMAIN, LogLevelFlags.LEVEL_ERROR, format, args);
	}
}
